/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef CieXyz_h
#define CieXyz_h


#include "Primitives.h"
#include "Vector3f.h"

#include "hxa7241_utility.h"




#include "hxa7241_graphics.h"
namespace hxa7241 { namespace graphics {
	using namespace hxa7241::primitives;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


/**
 * a CIE XYZ absolute light value.<br/><br/>
 *
 * essentially just a vector of 3 floats, this type enforces a semantic condition.
 */
class CieXyz
 :	public Vector3f
{
/// standard object services -----------------------------------------------------------------------
public:
	         CieXyz();
	         CieXyz( float x, float y, float z );
	explicit CieXyz( const float xyz[3] );
	explicit CieXyz( const XyzE& );
	explicit CieXyz( const Vector3f& );

	        ~CieXyz();
	         CieXyz( const CieXyz& );
	CieXyz&  operator=( const CieXyz& );

	CieXyz&  operator=( const XyzE& );
	CieXyz&  operator=( const Vector3f& );


/// commands ---------------------------------------------------------------------------------------
	        void fromXyzE( XyzE );


/// queries ----------------------------------------------------------------------------------------
	        XyzE toXyzE() const;


/// constants --------------------------------------------------------------------------------------
	static const CieXyz& ZERO();
	static const CieXyz& ONE();
	static const CieXyz& EPSILON();
	static const CieXyz& ALMOST_ONE();
	static const CieXyz& MIN();
	static const CieXyz& MAX();


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const CieXyz& );
	friend  SerializerIn&  operator>>( SerializerIn&,        CieXyz& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();
};








/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const CieXyz& );
SerializerIn&  operator>>( SerializerIn&,        CieXyz& );




/// INLINES ////


/// standard object services -----------------------------------------------------------------------
inline
CieXyz::CieXyz()
 :	Vector3f()
{
}


inline
CieXyz::CieXyz( float x, float y, float z )
 :	Vector3f( x, y, z )
{
}


inline
CieXyz::CieXyz( const float xyz[3] )
 :	Vector3f( xyz )
{
}


inline
CieXyz::CieXyz( const XyzE& xyze )
 :	Vector3f()
{
	fromXyzE( xyze );
}


inline
CieXyz::CieXyz( const Vector3f& v )
 :	Vector3f( v )
{
}


inline
CieXyz::~CieXyz()
{
}


inline
CieXyz::CieXyz( const CieXyz& other )
 :	Vector3f( other )
{
}


inline
CieXyz& CieXyz::operator=( const CieXyz& other )
{
	Vector3f::operator=( other );

	return *this;
}


inline
CieXyz& CieXyz::operator=( const XyzE& xyze )
{
	fromXyzE( xyze );

	return *this;
}


inline
CieXyz& CieXyz::operator=( const Vector3f& v )
{
	Vector3f::operator=( v );

	return *this;
}




/// constants --------------------------------------------------------------------------------------
inline
const CieXyz& CieXyz::ZERO()
{
	return (CieXyz&)Vector3f::ZERO();
}


inline
const CieXyz& CieXyz::ONE()
{
	return (CieXyz&)Vector3f::ONE();
}


inline
const CieXyz& CieXyz::EPSILON()
{
	return (CieXyz&)Vector3f::EPSILON();
}


inline
const CieXyz& CieXyz::ALMOST_ONE()
{
	return (CieXyz&)Vector3f::ALMOST_ONE();
}


inline
const CieXyz& CieXyz::MIN()
{
	return (CieXyz&)Vector3f::MIN();
}


inline
const CieXyz& CieXyz::MAX()
{
	return (CieXyz&)Vector3f::MAX();
}


}}




#endif//CieXyz_h
