/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Transclip_h
#define Transclip_h


#include "Primitives.h"
#include "Transform.h"
#include "Matrix4f.h"




#include "hxa7241_graphics.h"
namespace hxa7241 { namespace graphics {
	using namespace hxa7241::primitives;


/**
 * a transform from world space to clip space.
 */
class Transclip
	: public Transform
{
/// standard object services -----------------------------------------------------------------------
public:
	         Transclip();
	         Transclip( const Matrix4f& worldToClip );

	virtual ~Transclip();
	         Transclip( const Transclip& );
	Transclip& operator=( const Transclip& );


/// commands ---------------------------------------------------------------------------------------
	virtual Transclip& operator+=( const Transclip& );
	virtual Transclip& operator+=( const Matrix3f& );

	virtual Transclip& invert();


/// queries ----------------------------------------------------------------------------------------
	virtual void  transform( Vector3f& )                                                      const;
	virtual void  transform( UnitVector3f& )                                                  const;

	virtual bool  getClipping( const ClipFlags& enclosingClipFlags,
	                           const Vector3f   boundCorners[8],
	                           ClipFlags&       clipFlags )                                   const;
	virtual const Matrix4f& getMatrix()                                                       const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const Transclip& );


/// fields -----------------------------------------------------------------------------------------
private:
	Matrix4f matrix_m;
};


}}




#endif//Transclip_h
