/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Transplace_h
#define Transplace_h


#include "Primitives.h"
#include "Transform.h"
#include "Matrix3f.h"

#include "hxa7241_utility.h"




#include "hxa7241_graphics.h"
namespace hxa7241 { namespace graphics {
	using namespace hxa7241::primitives;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


/**
 * a rigid transform, of just translation and rotation.
 */
class Transplace
	: public Transform
{
/// standard object services -----------------------------------------------------------------------
public:
	         Transplace();
	         //Transplace( float scale, const Vector3f& rotationDegrees, const Vector3f& position );
	         Transplace( const Vector3f& rotationDegrees, const Vector3f& position );
	         Transplace( const Transplace&, bool invert );

	virtual ~Transplace();
	         Transplace( const Transplace& );
	Transplace& operator=( const Transplace& );


/// commands ---------------------------------------------------------------------------------------
public:
	//virtual void setTransplace( float scale, const Vector3f& rotationDegrees, const Vector3f& position );
	virtual void setTransplace( const Vector3f& rotationDegrees, const Vector3f& position );
	virtual void setTransplace( const Quaternion& rotation, const Vector3f& position );
	virtual void setTransplace( const Transplace&, bool invert );

	virtual Transplace& invert();
	virtual Transplace& operator+=( const Transplace& );


/// queries ----------------------------------------------------------------------------------------
public:
	//virtual float            getScale()         const;
	virtual const Vector3f&  getRotation()      const;
	virtual const Vector3f&  getPosition()      const;
	virtual const Matrix3f&  getMatrix()        const;

	virtual void  transform( Vector3f& )        const;
	virtual void  transform( UnitVector3f& )    const;

	virtual void  transform( float& )           const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const Transplace& );
	friend  SerializerIn&  operator>>( SerializerIn&,        Transplace& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const Transplace& );


/// fields -----------------------------------------------------------------------------------------
private:
	Matrix3f matrix_m;
	//float    scaling_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const Transplace& );
SerializerIn&  operator>>( SerializerIn&,        Transplace& );


}}




#endif//Transplace_h
