/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "RenderingOptions.h"
#include "RenderingState.h"
#include "SamplingControl.h"
#include "ImageAccumulation.h"

#include "RenderKernel.h"   /// own header is included last


using namespace perceptuum2::control;




/// standard object services -----------------------------------------------------------------------
RenderKernel::RenderKernel
(
	const Scene&            scene,
	const RenderingOptions& options,
	RenderingState&         state
)
 :	accumulation_m   ( state.getImageAccumulation() ),
	samplingControl_m( state.getSamplingControl() ),
	sceneIndex_m     ( scene ),
	illumination_m   ( options.getEmitterPathNumber(),
	                   options.getPhotonMapSize(),
	                   options.getLightsourceEmittanceThreshold(),
	                   options.getEmitterPathLengthLimit(),
	                   options.getEmitterPathEndThreshold(),
	                   options.getIsIlluminationLogging() ),
	rasterizer_m     ( options.getProjection(),
	                   options.getImageWidth(),
	                   options.getImageHeight(),
	                   options.getEyePathLengthLimit(),
	                   options.getEyePathEndThreshold(),
	                   options.getIsRasterizerLogging() )
{
}


RenderKernel::~RenderKernel()
{
}




/// commands ---------------------------------------------------------------------------------------
void RenderKernel::renderFrame()
{
	if( !isCompleted() )
	{
		sceneIndex_m.update( samplingControl_m );

		illumination_m.makeIllumination( sceneIndex_m, samplingControl_m );

		rasterizer_m.rasterizeFrame( samplingControl_m, sceneIndex_m, illumination_m, accumulation_m );

		accumulation_m.analyse();

		samplingControl_m.advanceFrame();
	}
}




/// queries ----------------------------------------------------------------------------------------
bool RenderKernel::isCompleted() const
{
	return accumulation_m.isCompleted();
}


float RenderKernel::getCompletedRatio() const
{
	return accumulation_m.getCompletedRatio();
}


dword RenderKernel::getFrameCount() const
{
	return samplingControl_m.getFrameIndex();
}
