/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderingOptions_h
#define RenderingOptions_h


#include "Primitives.h"
#include "RenderingOptionsImmutable.h"
#include "DisplayMeasures.h"
#include "CieXyz.h"
#include <string>

#include "hxa7241_utility.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::imaging::DisplayMeasures;
	using hxa7241::graphics::CieXyz;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::string;


class RenderingOptions
{
/// standard object services -----------------------------------------------------------------------
public:
	                  RenderingOptions();

	                 ~RenderingOptions();
private:
	                  RenderingOptions( const RenderingOptions& );
	RenderingOptions& operator=( const RenderingOptions& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void    setToDefault();

	        void    setImmutableOptions( const RenderingOptionsImmutable& );


/// queries ----------------------------------------------------------------------------------------
public:
	        void    getSummary( string& )                                                     const;

			/// constant for render
	        const RenderingOptionsImmutable& getImmutableOptions()                            const;
	        const Projection& getProjection()                                                 const;
	        dword   getImageWidth()                                                           const;
	        dword   getImageHeight()                                                          const;
	        CieXyz& getDefaultObjectColor( CieXyz& )                                          const;

			/// constant for session
	        dword   getPersistPeriod()                                                        const;
	        float   getTonemapping()                                                          const;

	        dword   getEmitterPathNumber()                                                    const;
	        dword   getPhotonMapSize()                                                        const;

	        float   getLightsourceEmittanceThreshold()                                        const;

	        dword   getEmitterPathLengthLimit()                                               const;
	        float   getEmitterPathEndThreshold()                                              const;
	        dword   getEyePathLengthLimit()                                                   const;
	        float   getEyePathEndThreshold()                                                  const;

	        void    getDisplayCharacteristics( DisplayMeasures& )                             const;

	        bool    getIsIlluminationLogging()                                                const;
	        bool    getIsRasterizerLogging()                                                  const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const RenderingOptions& );
	friend  SerializerIn&  operator>>( SerializerIn&,        RenderingOptions& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
private:
	        void  enforceConstraints();


/// fields -----------------------------------------------------------------------------------------
private:
	/// constant for render
	RenderingOptionsImmutable immutableOptions_m;

	/// constant for session
	dword secondsBetweenSaves_m;
	float tonemapAdaptationLuminance_m;

	dword emitterPathNumber_m;
	dword photonMapSize_m;

	float lightsourceEmittanceThreshold_m;

	dword emitterPathLengthLimit_m;
	float emitterPathEndThreshold_m;
	dword eyePathLengthLimit_m;
	float eyePathEndThreshold_m;

	DisplayMeasures displayCharacteristics_m;

	bool isIlluminationLogging_m;
	bool isRasterizerLogging_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const RenderingOptions& );
SerializerIn&  operator>>( SerializerIn&,        RenderingOptions& );


}}




#endif//RenderingOptions_h
