/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Serialization.h"
#include "TextSerializerOut.h"
#include <sstream>

#include "RenderingOptionsImmutable.h"


using namespace perceptuum2::control;




/// standard object services -----------------------------------------------------------------------
RenderingOptionsImmutable::RenderingOptionsImmutable()
{
	setToDefault();
}


RenderingOptionsImmutable::~RenderingOptionsImmutable()
{
}


RenderingOptionsImmutable::RenderingOptionsImmutable( const RenderingOptionsImmutable& other )
{
	RenderingOptionsImmutable::operator=( other );
}


RenderingOptionsImmutable& RenderingOptionsImmutable::operator=( const RenderingOptionsImmutable& other )
{
	if( &other != this )
	{
		projection_m         = other.projection_m;
		imageWidth_m         = other.imageWidth_m;
		imageHeight_m        = other.imageHeight_m;
		defaultObjectColor_m = other.defaultObjectColor_m;
	}

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
void RenderingOptionsImmutable::setToDefault()
{
	projection_m.setToDefault();
	imageWidth_m         = 624;
	imageHeight_m        = 400;
	defaultObjectColor_m = CieXyz::ZERO();
}


void RenderingOptionsImmutable::setImageSize
(
	const dword width,
	const dword height
)
{
	imageWidth_m  = width;
	imageHeight_m = height;
}




/// queries ----------------------------------------------------------------------------------------
void RenderingOptionsImmutable::getSummary( string& s ) const
{
	std::ostringstream oss;

	hxa7241::utility::TextSerializerOut::serializeOut( *this, oss );

	s = oss.str();
}


const Projection& RenderingOptionsImmutable::getProjection() const
{
	return projection_m;
}


dword RenderingOptionsImmutable::getImageWidth() const
{
	return imageWidth_m;
}


dword RenderingOptionsImmutable::getImageHeight() const
{
	return imageHeight_m;
}


CieXyz& RenderingOptionsImmutable::getDefaultObjectColor( CieXyz& c ) const
{
	return c = defaultObjectColor_m;
}




/// implementation ---------------------------------------------------------------------------------
void RenderingOptionsImmutable::enforceConstraints()
{
	imageWidth_m  = imageWidth_m  > 0 ? imageWidth_m  : 1;
	imageHeight_m = imageHeight_m > 0 ? imageHeight_m : 1;
}




/// serialization ----------------------------------------------------------------------------------
SerializerOut& perceptuum2::control::operator<<( SerializerOut& out, const RenderingOptionsImmutable& obj )
{
	out.write( "projection",         obj.projection_m         );
	out.write( "imageWidth",         obj.imageWidth_m         );
	out.write( "imageHeight",        obj.imageHeight_m        );
	out.write( "defaultObjectColor", obj.defaultObjectColor_m );

	return out;
}


SerializerIn& perceptuum2::control::operator>>( SerializerIn& in, RenderingOptionsImmutable& obj )
{
	in.read( "projection",         obj.projection_m         );
	in.read( "imageWidth",         obj.imageWidth_m         );
	in.read( "imageHeight",        obj.imageHeight_m        );
	in.read( "defaultObjectColor", obj.defaultObjectColor_m );

	obj.enforceConstraints();

	return in;
}


const char* RenderingOptionsImmutable::CLASS_NAME()
{
	static const char CLASS_NAME[] = "perceptuum2::control::RenderingOptionsImmutable";
	return CLASS_NAME;
}


const char* RenderingOptionsImmutable::CLASS_DESCRIPTION()
{
	static const char CLASS_DESCRIPTION[] = "options for controlling a rendering that are immutable across sessions";
	return CLASS_DESCRIPTION;
}
