/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef GammaTransform_h
#define GammaTransform_h


#include "Primitives.h"
#include "Vector3f.h"
#include "Array.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::utility::Array;


class GammaTransform
{
/// standard object services -----------------------------------------------------------------------
public:
	         GammaTransform();
	         GammaTransform( const Vector3f& gammaXyz );

	virtual ~GammaTransform();
	         GammaTransform( const GammaTransform& );
	GammaTransform& operator=( const GammaTransform& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  setSRGB();
	virtual void  set( const Vector3f& gammaXyz );


/// queries ----------------------------------------------------------------------------------------
public:
	static  float gammaEncode( float fp01c, float invGamma );
	virtual float gammaEncode( float fp01c )                                                  const;
	virtual void  gammaEncodeQuantize255( const Vector3f& colorFp, dword color24[3] )         const;

	static  float gammaDecode( float fp01c, float gamma );
	virtual float gammaDecode( float fp01c )                                                  const;
	virtual void  gammaDecode( const Vector3f& in, Vector3f& out )                            const;
	//virtual void  gammaDecodeFractionalize255( const dword color24[3] color24,
	//                                           Vector3f& colorFp )                            const;

	virtual void  getGamma( Vector3f& gammaXyz )                                              const;
	virtual void  getInvGamma( Vector3f& gammaXyz )                                           const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  setGamma( const Vector3f& gammaXyz );
	virtual void  getFromTable( const Array<byte>& table, const Vector3f& colorFp,
	                            dword color24[3] )                                            const;


/// fields -----------------------------------------------------------------------------------------
private:
	Vector3f    gamma_m;
	Vector3f    invGamma_m;

	Array<byte> gammaTable_m;
	Array<byte> invGammaTable_m;

	static const dword    TABLE_LENGTH = 2048;   /// must be power of 2
	static const Vector3f sRGB_GAMMA;
};


}}




#endif//GammaTransform_h
