/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ImageAccumulation_h
#define ImageAccumulation_h


#include "Primitives.h"
#include "PixelStats.h"
#include "Sheet.h"
#include <vector>

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::utility::Sheet;
	using hxa7241::utility::RandomGood;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


class ImageAccumulation
{
/// standard object services -----------------------------------------------------------------------
public:
	         ImageAccumulation();

	        ~ImageAccumulation();
	         ImageAccumulation( const ImageAccumulation& );
	ImageAccumulation& operator=( const ImageAccumulation& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  setImage( ImageCieXyz* );

	        bool  shouldSample( dword x,
	                            dword y );
	        void  addSample( dword x,
	                         dword y,
	                         float xOffset01,
	                         float yOffset01,
	                         const CieXyz&,
	                         float alpha );

	        void  analyse();


/// queries ----------------------------------------------------------------------------------------
public:
	        dword getPixelCount()           const;
	        dword getWidth()                const;
	        dword getHeight()               const;

	        dword getSamplesCount()         const;
	        dword getSamplesRequiredCount() const;
	        float getCompletedRatio()       const;
	        bool  isCompleted()             const;

	        float getMaxDeviation()         const;


/// implementation ---------------------------------------------------------------------------------
protected:
	        void  analyseSpatialDeviation( const ImageCieXyz&,
	                                       Sheet<PixelStats>& )                               const;
	        void  analyseSamplesRequired( const ImageCieXyz&,
	                                      const Sheet<PixelStats>&,
	                                      vector<bool>&            required,
	                                      bool&                    isAnySamplesRequired,
	                                      dword&                   requiredCount,
	                                      float&                   completedRatio )           const;

	        bool  checkSheets( const char methodName[] ) const;


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const ImageAccumulation& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ImageAccumulation& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	ImageCieXyz*      pImage_m;
	Sheet<PixelStats> pixelStats_m;
	vector<bool>      requiredSamples_m;
	bool              isAnySamplesRequired_m;

	dword samplesCount_m;
	dword samplesRequiredCount_m;
	float completedRatio_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ImageAccumulation& );
SerializerIn&  operator>>( SerializerIn&,        ImageAccumulation& );


}}




#endif//ImageAccumulation_h
