/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ImageCieXyz_h
#define ImageCieXyz_h


#include "Primitives.h"
#include "CieXyz.h"
#include "Sheet.h"
#include <string>

#include "hxa7241_utility.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::utility::Sheet;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::string;


class ImageCieXyz
 :	public Sheet<CieXyz>
{
/// standard object services -----------------------------------------------------------------------
public:
	             ImageCieXyz();
	             ImageCieXyz( dword width, dword height );                    /// throws

	            ~ImageCieXyz();
	             ImageCieXyz( const ImageCieXyz& );
	ImageCieXyz& operator=( const ImageCieXyz& );


/// commands ---------------------------------------------------------------------------------------
	        void  setSize( dword width, dword height );                  /// throws

	        void  zeroMemory()                                     const;
	        void  setAlphaOpaque()                                 const;


/// queries ----------------------------------------------------------------------------------------
	        byte* getAlphaMemory()                                 const;
	        byte* getAlphaRow( dword y )                           const;
	        byte& getAlphaElement( dword x, dword y )              const;

	        void  getSummary( string& )                            const;

	        void  getLuminanceMeans( float& mean, float& logMean ) const;
	        void  getLuminanceRange( float& min,  float& max )     const;
	        float getLogMeanLuminance()                            const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const ImageCieXyz& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ImageCieXyz& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	Sheet<byte> alpha_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ImageCieXyz& );
SerializerIn&  operator>>( SerializerIn&,        ImageCieXyz& );


}}




#endif//ImageCieXyz_h
