/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ImageConverter_h
#define ImageConverter_h


#include "Primitives.h"
#include "ToneMapperWard94.h"
#include "ColorTransformXyzToRgbFp.h"
#include "GammaTransform.h"

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::graphics::Vector3f;


class ImageConverter
{
/// standard object services -----------------------------------------------------------------------
public:
	                ImageConverter();
	                ImageConverter( const CieXyz& whiteColor,
	                                const Vector3f chromaticitiesXyz[3], const Vector3f& gammaXyz );
	                ImageConverter( const CieXyz& blackPoint, const CieXyz& whitePoint,
	                                const Vector3f chromaticitiesXyz[3], const Vector3f& gammaXyz );

	virtual        ~ImageConverter();
private:
	                ImageConverter( const ImageConverter& );
	ImageConverter& operator=( const ImageConverter& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  setSRGB();
	virtual void  set( const CieXyz& whiteColor, const Vector3f chromaticitiesXyz[3] );
	virtual void  set( const CieXyz& blackPoint, const CieXyz& whitePoint,
	                   const Vector3f chromaticitiesXyz[3] );
	virtual void  set( const Vector3f& gammaXyz );
	virtual void  set( float adaptLuminanceCdPerM2 );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void  convertXYZtoRGB( const ImageCieXyz&, bool reAssociateAlpha,
	                               ImageRgba& )                                               const;
	virtual void  convertXYZtoRGB( const ImageCieXyz&, dword row, bool reAssociateAlpha,
	                               ImageRgba& )                                               const;
	virtual void  convertRGBtoXYZ( const ImageRgba&, bool isRgbAlphaAssociated, bool isTonemapable,
	                               ImageCieXyz& )                                             const;
	virtual void  convertRGBtoXYZ( const Vector3f& rgb, byte alpha, bool isRgbAlphaAssociated,
	                               bool isTonemapable,
	                               CieXyz& xyz )                                              const;

	virtual float getAdaptLuminance()                                                         const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  checkImageDimensionMatch( const ImageCieXyz&, const ImageRgba& )            const;

	virtual void  convertXYZtoRGB( const CieXyz&, dword rgb[3] )                              const;
	virtual void  convertRGBtoXYZ( const dword rgba[4], bool isRgbAlphaAssociated,
	                               bool isTonemapable,
	                               CieXyz& )                                                  const;


/// fields -----------------------------------------------------------------------------------------
private:
	ToneMapperWard94         toneMapper_m;
	ColorTransformXyzToRgbFp colorConverter_m;
	GammaTransform           gammaTransformer_m;

	static const char CLASS_NAME[];
	static const char EXCEPTION_MESSAGE1[];
};


}}




#endif//ImageConverter_h
