/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Float16.h"
#include "Serialization.h"

#include "PixelStats.h"   /// own header is included last


using namespace perceptuum2::imaging;




/// standard object services -----------------------------------------------------------------------
PixelStats::PixelStats()
 :	variance_m( 0.0f ),
	count_m( 0.0f )
{
}


PixelStats::~PixelStats()
{
}


PixelStats::PixelStats( const PixelStats& other )
{
	PixelStats::operator=( other );
}


PixelStats& PixelStats::operator=( const PixelStats& other )
{
	variance_m = other.variance_m;
	count_m    = other.count_m;

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
void PixelStats::setCount( const float f )
{
	count_m = f;
}


void PixelStats::incCount()
{
	count_m += 1.0f;
}


void PixelStats::setVariance( const float f )
{
	variance_m = f;
}




/// queries ----------------------------------------------------------------------------------------
float PixelStats::getCount() const
{
	return count_m;
}


float PixelStats::getVariance() const
{
	return variance_m;
}




/// implementation ---------------------------------------------------------------------------------
void PixelStats::enforceConstraints()
{
	variance_m = variance_m >= 0.0f ? variance_m : 0.0f;
	count_m    = count_m    >= 0.0f ? count_m    : 0.0f;
}




/// serialization ----------------------------------------------------------------------------------
SerializerOut& perceptuum2::imaging::operator<<( SerializerOut& out, const PixelStats& obj )
{
	out.write( "variance", obj.variance_m );
	out.write( "count",    obj.count_m    );

	return out;
}


SerializerIn& perceptuum2::imaging::operator>>( SerializerIn& in, PixelStats& obj )
{
	in.read( "variance", obj.variance_m );
	in.read( "count",    obj.count_m    );

	obj.enforceConstraints();

	return in;
}


const char* PixelStats::CLASS_NAME()
{
	static const char CLASS_NAME[] = "perceptuum2::imaging::PixelStats";
	return CLASS_NAME;
}


const char* PixelStats::CLASS_DESCRIPTION()
{
	static const char CLASS_DESCRIPTION[] = "sampling stats for a pixel";
	return CLASS_DESCRIPTION;
}
