/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef PngReader_h
#define PngReader_h


#include "Primitives.h"
#include "ImageReader.h"
#include "Vector3f.h"
#include <string>
#include <iosfwd>

#include "hxa7241_graphics.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::graphics::Vector3f;
	using std::string;
	using std::istream;


class PngReader
 :	public ImageReader
{
/// standard object services -----------------------------------------------------------------------
public:
	         PngReader() {}

	virtual ~PngReader() {}
private:
	         PngReader( const PngReader& );
	PngReader& operator=( const PngReader& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void getName( string& )                                                           const;
	/// returns whether the stream was recognized (and therefore also read)
	virtual bool read( istream& imageBytes, float adaptLuminance, ImageCieXyz& )              const;   /// for reading all color images
	virtual bool read( istream& imageBytes, ImageRgba&, ImageConverter& )                     const;   /// for reading all color images
	virtual bool read( istream& imageBytes, Sheet<byte>& )                                    const;   /// for reading grayscale images

	static const char* CLASS_NAME();

	static const char EXCEPTION_MESSAGE_1[];
	static const char EXCEPTION_MESSAGE_2[];
	static const char EXCEPTION_MESSAGE_3[];


/// implementation ---------------------------------------------------------------------------------
private:
	enum EConversion { COLOR = 1, GAMMA = 2, SRGB = 4 };

	virtual bool readPng( istream& imageBytes, Sheet<byte>*, ImageRgba*, dword& conversionFlags,
	                      CieXyz& whitePoint, Vector3f chromaticitiesXyz[3], Vector3f& gammaXyz )   const;


/// fields -----------------------------------------------------------------------------------------
private:

	static const char FORMAT_NAME[];
};


}}




#endif//PngReader_h
