/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Bssdf_h
#define Bssdf_h


#include "Primitives.h"
#include "Brdf.h"




#include "perceptuum2_interaction.h"
namespace perceptuum2 { namespace interaction {
	using namespace hxa7241::primitives;


/**
 * transmission enhancement decorator for brdf (still abstract).<br/><br/>
 *
 * fakes the transmission distribution by using the reflective one on both sides.
 * <br/><br/>
 *
 * superluminal phase velocity is not supported.
 */
class Bssdf
	: public Brdf
{
/// standard object services -----------------------------------------------------------------------
protected:
	         Bssdf();
	         Bssdf( const UnitVector3f& normal,
	                const UnitVector3f& tangent,
	                float               refractiveIndexOutside,
	                float               refractiveIndexInside,
	                Float01             transparency );
public:

	virtual ~Bssdf();
protected:
	         Bssdf( const Bssdf& );
	Bssdf& operator=( const Bssdf& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  setBssdf( float   refractiveIndexOutside,
	                        float   refractiveIndexInside,
	                        Float01 transparency );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void    getScaling( dword               part,
	                            const UnitVector3f& inDirection,
	                            const UnitVector3f& outDirection,
	                            Light&              scaling )                              const;
	/** @return is transmitted */
	virtual bool    getSample( dword               part,
	                           Float01             roulette,
	                           const Vector2f01&   stochastic,
	                           const UnitVector3f& inDirection,
	                           UnitVector3f&       outDirection,
	                           Light&              scaling )                               const;


	virtual void  getDirectionFromCoords( const Vector2f01& coords,
	                                      bool              isUpper,
	                                      UnitVector3f&     direction )                    const;
	virtual void  getCoordsFromDirection( const UnitVector3f& direction,
	                                      Vector2f01&         coords,
	                                      bool&               isUpper )                    const;


/// fields -----------------------------------------------------------------------------------------
private:
	float   refractiveIndexOutside_m;
	float   refractiveIndexInside_m;
	Float01 transparency_m;

	float   relativeRefractiveIndex_m;
};


}}




#endif//Bssdf_h
