/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Light_h
#define Light_h


#include "Primitives.h"
#include <iosfwd>

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_interaction.h"
namespace perceptuum2 { namespace interaction {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


class Light
{
/// standard object services -----------------------------------------------------------------------
public:
	         Light();
	explicit Light( const CieXyz& );

	        ~Light();
	         Light( const Light& );
	Light& operator=( const Light& );

	Light& operator=( const CieXyz& );

private:
	         Light( float s0, float s1, float s2 );

	friend  std::ostream& operator<<( std::ostream&, const Light& );
	friend  std::istream& operator>>( std::istream&,       Light& );


/// commands ---------------------------------------------------------------------------------------
public:
	        Light& setToMean1Color();
	        Light& clampToPositive();
	        Light& clampTo01Positive();


/// queries ----------------------------------------------------------------------------------------
	        CieXyz getXYZ()                              const;
	        void   getXYZ( CieXyz& )                     const;
	        float  getLuminance()                        const;
	        void   getMeanAndColor( float& mean,
	                                Light& color )       const;

	        float  total()                               const;
	        float  average()                             const;
	        float  smallest()                            const;
	        float  largest()                             const;


/// arithmetic -------------------------------------------------------------------------------------
	        Light  operator+ ( const Light& )            const;
	        Light& operator+=( const Light& );
	        Light  operator- ( const Light& )            const;
	        Light& operator-=( const Light& );

	        Light  operator* ( const Light& )            const;
	        Light& operator*=( const Light& );
	        Light  operator/ ( const Light& )            const;
	        Light& operator/=( const Light& );

	        Light  operator* ( float )                   const;
	        Light& operator*=( float );
	        Light  operator/ ( float )                   const;
	        Light& operator/=( float );

	friend  Light  operator*( float, const Light& );
	friend  Light  operator/( float, const Light& );


/// logical ---------------------------------------------------------------------------------------
	        bool   operator==( const Light& )            const;
	        bool   operator!=( const Light& )            const;
	        bool   isZero()                              const;


/// constants --------------------------------------------------------------------------------------
	static const Light& ZERO();
	static const Light& ONE();
	static const Light& EPSILON();
	static const Light& ALMOST_ONE();
	static const Light& MIN();
	static const Light& MAX();


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const Light& );
	friend  SerializerIn&  operator>>( SerializerIn&,        Light& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	static const int SIZE = 3;
	float s_m[ SIZE ];
};








/// friends
Light operator*( float, const Light& );
Light operator/( float, const Light& );




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const Light& );
SerializerIn&  operator>>( SerializerIn&,        Light& );

std::ostream& operator<<( std::ostream&, const Light& );
std::istream& operator>>( std::istream&,       Light& );




/// INLINES ////


/// standard object services -----------------------------------------------------------------------
inline
Light::Light( const CieXyz& xyz )
{
	Light::operator=( xyz );
}


inline
Light::Light( const Light& other )
{
	Light::operator=( other );
}


inline
Light::~Light()
{
}


}}




#endif//Light_h
