/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Ray_h
#define Ray_h


#include "Primitives.h"
#include "Vector3f.h"
#include "UnitVector3f.h"
#include "ObjectRenderableId.h"
#include <iosfwd>


#include "hxa7241_graphics.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Transform;


class Ray
{
/// standard object services -----------------------------------------------------------------------
public:
	         Ray();
	         //Ray( const Vector3f& position, const UnitVector3f& direction );
	         //Ray( const Vector3f& position, const UnitVector3f& direction,
	         //     const UnitVector3f& normal, const ObjectRenderableId& object, bool isTransmitted );
	         //Ray( const Vector3f& position, const UnitVector3f& direction,
	         //     const UnitVector3f& normal, const ObjectRenderableId& object );

	        ~Ray();
	         Ray( const Ray& );
	Ray& operator=( const Ray& );

	friend  std::ostream& operator<<( std::ostream&, const Ray& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  set( const Vector3f& position, const UnitVector3f& direction );
	        void  set( const Vector3f& position, const UnitVector3f& direction,
	                   const UnitVector3f& normal, const ObjectRenderableId& object, bool isTransmitted );
	        void  set( const Vector3f& position, const UnitVector3f& direction,
	                   const UnitVector3f& normal, const ObjectRenderableId& object );
	        //void  setPosition( const Vector3f& );
	        //void  setDirection( const UnitVector3f& );
	        //void  setNormal( const UnitVector3f& );
	        //void  setObject( const ObjectRenderableId& );

	        Ray&  transformEq( const Transform& );


/// queries ----------------------------------------------------------------------------------------
public:
	        const Vector3f&           getPosition()                                           const;
	        const UnitVector3f&       getDirection()                                          const;
	        const UnitVector3f&       getNormal()                                             const;
	        const ObjectRenderableId& getObject()                                             const;

	        bool  isTransmission()                                                            const;
	        bool  isInside()                                                                  const;

	        void     getPosition( float distance, Vector3f& )                                 const;
	        Vector3f getPosition( float distance )                                            const;


/// implementation ---------------------------------------------------------------------------------
private:
	        bool  isDirDotNormNeg()    const;
	        bool  calcIsTransmission() const;


/// fields -----------------------------------------------------------------------------------------
private:
	Vector3f     position_m;
	UnitVector3f direction_m;

	ObjectRenderableId objectLastHit_m;
	UnitVector3f       normalOfLastHit_m;
	UnitVector3f       lastDirection_m;

	/// auxilliary
	bool isInside_m;
	bool isTransmission_m;
};








std::ostream& operator<<( std::ostream&, const Ray& );




/// INLINES ///


/// queries ----------------------------------------------------------------------------------------
inline
const Vector3f& Ray::getPosition() const
{
	return position_m;
}


inline
const UnitVector3f& Ray::getDirection() const
{
	return direction_m;
}


inline
const UnitVector3f& Ray::getNormal() const
{
	return normalOfLastHit_m;
}


inline
const ObjectRenderableId& Ray::getObject() const
{
	return objectLastHit_m;
}


}}




#endif//Ray_h
