/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef TriangleExplicit_h
#define TriangleExplicit_h


#include "Primitives.h"
#include "Vertex.h"
#include "Vector3f.h"
#include "Vector2.h"

#include "hxa7241_graphics.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Vector2;
	using hxa7241::graphics::Vector2f;
	using hxa7241::utility::Float01;


class TriangleExplicit
{
/// standard object services -----------------------------------------------------------------------
public:
	         TriangleExplicit();
	         TriangleExplicit( const Vertex&   vert0,
	                           const Vertex&   vert1,
	                           const Vertex&   vert2,
	                           const Vector3f& position,
	                           const Vector3f& scaling,
	                           bool            interpolateNormal );

	        ~TriangleExplicit();
	         TriangleExplicit( const TriangleExplicit& );
	TriangleExplicit& operator=( const TriangleExplicit& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  set( const Vertex&   vert0,
	                   const Vertex&   vert1,
	                   const Vertex&   vert2,
	                   const Vector3f& position,
	                   const Vector3f& scaling,
	                   bool            interpolateNormal );


/// queries ----------------------------------------------------------------------------------------
public:
	        void  getVertexPosition0( Vector3f& )                                             const;
	        void  getVertexPosition1( Vector3f& )                                             const;
	        void  getVertexPosition2( Vector3f& )                                             const;

	        void  getVertexNormal0( UnitVector3f& )                                           const;
	        void  getVertexNormal1( UnitVector3f& )                                           const;
	        void  getVertexNormal2( UnitVector3f& )                                           const;

	        Vector3f getEdge( dword vertex0,
	                          dword vertex1 )                                                 const;

	        bool  getIntersection( const Ray&    ray,
	                               float&        distance,
	                               UnitVector3f& normal,
	                               Vector2f&     barycentrics )                               const;
	        bool  getBarycentrics( const Vector3f& position,
	                               Vector2f& )                                                const;
	        void  getPosition( const Vector2f& barycentrics,
	                           Vector3f& )                                                    const;
	        void  getNormal  ( const Vector2f& barycentrics,
	                           UnitVector3f& )                                                const;
	        void  getNormal  ( UnitVector3f& )                                                const;


/// fields -----------------------------------------------------------------------------------------
private:
	Vertex   vertex0_m;
	Vertex   vertex1_m;
	Vertex   vertex2_m;

	Vector3f position_m;
	Vector3f scaling_m;

	bool     interpolateNormal_m;
};


}}




#endif//TriangleExplicit_h
