/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Vertex_h
#define Vertex_h


#include "Primitives.h"

#include "hxa7241_graphics.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;


class Vertex
{
/// standard object services -----------------------------------------------------------------------
public:
	        Vertex();
	        /// ((position - meshPosition) * oneOverMeshScale) must map into the [-0.5, 0.5] interval
	        Vertex( const Vector3f& meshPosition, const Vector3f& oneOverMeshScale,
	                const Vector3f& position, const UnitVector3f& normal );

	       ~Vertex();
	        Vertex( const Vertex& );
	Vertex& operator=( const Vertex& );


/// commands ---------------------------------------------------------------------------------------
public:
	        /// ((position - meshPosition) * oneOverMeshScale) must map into the [-0.5, 0.5] interval
	        void  set( const Vector3f& meshPosition, const Vector3f& oneOverMeshScale,
	                   const Vector3f& position, const UnitVector3f& normal );


/// queries ----------------------------------------------------------------------------------------
public:
	        void  getPosition( const Vector3f& meshPosition, const Vector3f& meshScale,
	                           Vector3f& position )                                           const;
	        void  getNormal( UnitVector3f& normal )                                           const;


/// fields -----------------------------------------------------------------------------------------
private:
	/// compressed format (63 bits == 8 bytes)
	/// positionX : 14 bits
	/// positionY : 14 bits
	/// positionZ : 14 bits
	/// normalX   :  7 bits
	/// normalY   :  7 bits
	/// normalZ   :  7 bits
	/// position precision: 16m*16m*16m object accurate to mm
	/// normal precision:   0.74 degrees, 0.013 radians
	dword posXposYnormZlsb_m;
	dword posZnormXnormYnormZmsb_m;

	static const float POSITION_SCALING;
	static const float NORMAL_SCALING;
	static const float POSITION_SCALING_INV;
	static const float NORMAL_SCALING_INV;
};


}}




#endif//Vertex_h
