/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Camera_h
#define Camera_h


#include "Primitives.h"
#include "Vector2.h"
#include "Float01.h"

#include "hxa7241_graphics.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::Vector2;
	using hxa7241::graphics::Vector2f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Matrix4f;
	using hxa7241::utility::Float01;
	typedef Vector2<Float01> Vector2f01;


class Camera
{
/// standard object services -----------------------------------------------------------------------
public:
	         Camera( const Projection& projection,
	                 dword             imageWidth,
	                 dword             imageHeight );

	virtual ~Camera();
	         Camera( const Camera& );
	Camera& operator=( const Camera& );


/// queries ----------------------------------------------------------------------------------------
	virtual void  getRay( dword                  x,
	                      dword                  y,
	                      const SamplingControl& samplingControl,
	                      Vector2f01&            subPixelPosition,
	                      Vector3f&              start,
	                      UnitVector3f&          direction )                                  const;
	virtual void  getWorldToViewTransform( const SamplingControl&,
	                                       Matrix4f& )                                        const;
	virtual void  getViewToClipTransform( const SamplingControl&,
	                                      Matrix4f& )                                         const;
	virtual void  getClipToScreenScaling( Vector2f& )                                         const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const Camera& );


/// fields -----------------------------------------------------------------------------------------
private:
	const Projection* pProjection_m;
	dword imageWidth_m;
	dword imageHeight_m;

	float halfImageWidth_m;
	float halfImageHeight_m;
	float oneOverImageWidth_m;

	static const char CLASS_NAME[];
};


}}




#endif//Camera_h
