/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Projection_h
#define Projection_h


#include "Primitives.h"
#include "Vector2.h"
#include "Vector3f.h"
#include "UnitVector3f.h"

#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector2f;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Matrix4f;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


class Projection
{
/// standard object services -----------------------------------------------------------------------
public:
	         Projection();

	        ~Projection();
	         Projection( const Projection& );
	Projection& operator=( const Projection& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void setToDefault();


/// queries ----------------------------------------------------------------------------------------
public:
	        void  getRay( float                  x,
	                      float                  y,
	                      const SamplingControl& samplingControl,
	                      Vector3f&              start,
	                      UnitVector3f&          direction )                                  const;
	        void  getWorldToViewTransform( const SamplingControl&,
	                                       const Vector2f& subPixelPosition,
	                                       Matrix4f& )                                        const;
	        void  getViewToClipTransform( const SamplingControl&,
	                                      const Vector2f& oneOverWidthHeight,
	                                      Matrix4f& )                                         const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const Projection& );
	friend  SerializerIn&  operator>>( SerializerIn&,        Projection& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	        void  enforceConstraints();
	        void  setPreCalculations();


/// fields -----------------------------------------------------------------------------------------
private:
	/// serialized
	Vector3f     eyePosition_m;
	UnitVector3f lookDirection_m;
	UnitVector3f upDirection_m;
	float        width_m;
	float        screenDistance_m;
	float        lensDiameter_m;
	float        focalLength_m;

	/// auxilliary precalculations
	Vector3f     screenCenterVector_m;
	UnitVector3f sideDirection_m;

	/// constants
	static const UnitVector3f LOOKDIRECTION_DEFAULT;
	static const UnitVector3f UPDIRECTION_DEFAULT;
	static const float        WIDTH_DEFAULT;
	static const float        SCREENDISTANCE_DEFAULT;
	static const float        LENSDIAMETER_DEFAULT;
	static const float        FOCALLENGTH_DEFAULT;
	static const float        Z_NEAR_CLIP_PLANE;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const Projection& );
SerializerIn&  operator>>( SerializerIn&,        Projection& );


}}




#endif//Projection_h
