/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RasterizerTracing_h
#define RasterizerTracing_h


#include "Primitives.h"
#include "Camera.h"
#include "EyePath.h"

#include "perceptuum2_imaging.h"
#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using perceptuum2::imaging::ImageAccumulation;
	using hxa7241::utility::Float01c;


class RasterizerTracing
{
/// standard object services -----------------------------------------------------------------------
public:
	         RasterizerTracing( const Projection& projection,
	                            dword             imageWidth,
	                            dword             imageHeight,
	                            dword             eyePathLengthLimit,
	                            Float01c          eyePathEndThreshold,
	                            bool              isLogging );

	virtual ~RasterizerTracing();
private:
	         RasterizerTracing( const RasterizerTracing& );
	RasterizerTracing& operator=( const RasterizerTracing& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  rasterizeFrame( const SamplingControl&,
	                              const SpacetimeIndex&,
	                              const Illumination&,
	                              ImageAccumulation& );


/// queries ----------------------------------------------------------------------------------------
public:


/// fields -----------------------------------------------------------------------------------------
private:
	Camera  camera_m;
	EyePath eyePath_m;

	/// overall statistics
	dword totalEyePathCount_m;
	dword totalEyePathLengthCount_m;
	dword totalOverdrawCount_m;

	bool isLogging_m;
};


}}




#endif//RasterizerTracing_h
