/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Vector3f.h"
#include "Vector2.h"
#include "Float01.h"
#include "Serialization.h"

#include "SamplingControlEmitter.h"   /// own header is included last


using namespace perceptuum2::transport;




/// standard object services -----------------------------------------------------------------------
SamplingControlEmitter::SamplingControlEmitter
(
	const dword sobolDimensions0[SOBOL_COUNT],
	const dword sobolDimensions1[],
	const dword randomSeeds[],
	const dword specialLevels,
	const dword extraSeed
)
 :	pathControl_m( sobolDimensions1, randomSeeds, specialLevels, extraSeed )
{
	for( dword i = SOBOL_COUNT;  i-- > 0; )
	{
		sobols_m[i].setDimension( sobolDimensions0[i] );
	}
}


SamplingControlEmitter::~SamplingControlEmitter()
{
}


SamplingControlEmitter::SamplingControlEmitter
(
	const SamplingControlEmitter& other
)
 :	pathControl_m( other.pathControl_m )
{
	for( dword i = SOBOL_COUNT;  i-- > 0; )
	{
		sobols_m[i] = other.sobols_m[i];
	}
}


SamplingControlEmitter& SamplingControlEmitter::operator=
(
	const SamplingControlEmitter& other
)
{
	if( &other != this )
	{
		pathControl_m = other.pathControl_m;

		for( dword i = SOBOL_COUNT;  i-- > 0; )
		{
			sobols_m[i] = other.sobols_m[i];
		}
	}

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
void SamplingControlEmitter::advance()
{
	for( dword i = SOBOL_COUNT;  i-- > 0; )
	{
		sobols_m[i].next();
	}

	pathControl_m.nextPath();
}




/// queries ----------------------------------------------------------------------------------------
Vector3f SamplingControlEmitter::getPositionCoords() const
{
	return Vector3f( sobols_m[POS_X].getItem(),
	                 sobols_m[POS_Y].getItem(),
	                 sobols_m[POS_R].getItem() );
}


Vector2f01 SamplingControlEmitter::getDirectionCoords() const
{
	return Vector2<Float01>( sobols_m[DIR_X].getItem(),
	                         sobols_m[DIR_Y].getItem() );
}


void SamplingControlEmitter::getInteractionCoords
(
	dword   pathNodeIndex,
	Float01 coords[SamplingControlPath::COORDS_COUNT]
) const
{
	pathControl_m.getSampleCoords( pathNodeIndex, coords );
}




/// serialization ----------------------------------------------------------------------------------
SerializerOut& perceptuum2::transport::operator<<( SerializerOut& out, const SamplingControlEmitter& obj )
{
	for( dword i = 0;  i < obj.SOBOL_COUNT;  ++i )
	{
		out.write( "sobol", obj.sobols_m[i] );
	}

	out.write( "pathControl", obj.pathControl_m );

	return out;
}


SerializerIn& perceptuum2::transport::operator>>( SerializerIn& in, SamplingControlEmitter& obj )
{
	for( dword i = 0;  i < obj.SOBOL_COUNT;  ++i )
	{
		in.read( "sobol", obj.sobols_m[i] );
	}

	in.read( "pathControl", obj.pathControl_m );

	return in;
}


const char* SamplingControlEmitter::CLASS_NAME()
{
	static const char CLASS_NAME[] = "perceptuum2::transport::SamplingControlEmitter";
	return CLASS_NAME;
}


const char* SamplingControlEmitter::CLASS_DESCRIPTION()
{
	static const char CLASS_DESCRIPTION[] = "grouping of stochastic sequences used in emitter sampling";
	return CLASS_DESCRIPTION;
}
