/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef SamplingControlPath_h
#define SamplingControlPath_h


#include "Primitives.h"
#include "SobolSequence.h"
#include "RandomGood.h"
#include "Float01.h"
#include <vector>

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::utility::SobolSequence;
	using hxa7241::utility::RandomGood;
	using hxa7241::utility::Float01;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


/**
 * path interaction sampling controller.<br/><br/>
 *
 * provides sampling coordinates for all interactions along a path.<br/>
 * the first two path node coords are sobol generated (partly), the rest are
 * pseudo random.
 * <br/><br/>
 */
class SamplingControlPath
{
public:
	static const dword SOBOL_COUNT  = 4;
	static const dword RANDOM_COUNT = 4;
	static const dword COORDS_COUNT = 8;

/// standard object services -----------------------------------------------------------------------
public:
	         /** @param sobolDimensions must have length == SOBOL_COUNT * specialLevels
	             @param randomSeeds must have length == RANDOM_COUNT * specialLevels */
	         SamplingControlPath( const dword sobolDimensions[],
	                              const dword randomSeeds[],
	                              dword       specialLevels,
	                              dword       extraSeed );

	        ~SamplingControlPath();
	         SamplingControlPath( const SamplingControlPath& );
	SamplingControlPath& operator=( const SamplingControlPath& );


/// commands ---------------------------------------------------------------------------------------
	        void  nextPath();


/// queries ----------------------------------------------------------------------------------------
	        void  getSampleCoords( dword   pathNodeIndex,
	                               Float01 coords[COORDS_COUNT] )                             const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const SamplingControlPath& );
	friend  SerializerIn&  operator>>( SerializerIn&,        SamplingControlPath& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	vector<SobolSequence> sobols_m;
	vector<RandomGood>    randoms_m;

	mutable RandomGood    laterRandom_m;
	mutable vector<float> laterCoords_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const SamplingControlPath& );
SerializerIn&  operator>>( SerializerIn&,        SamplingControlPath& );


}}




#endif//SamplingControlPath_h
