/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Projector_h
#define Projector_h


#include "Primitives.h"
#include "Matrix4f.h"
#include "Vector2.h"

#include "perceptuum2_transport.h"




#include "perceptuum2_transport_projection.h"
namespace perceptuum2 { namespace transport { namespace projection {
	using namespace hxa7241::primitives;
	using perceptuum2::transport::Camera;
	using perceptuum2::transport::SamplingControl;
	using hxa7241::graphics::Matrix4f;
	using hxa7241::graphics::ClipFlags;
	using hxa7241::graphics::Vector2f;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::Vector4f;
	using hxa7241::graphics::UnitVector3f;


class Projector
{
/// standard object services -----------------------------------------------------------------------
public:
	         Projector( const Camera&,
	                    const SamplingControl& );

	        ~Projector();
	         Projector( const Projector& );
	Projector& operator=( const Projector& );


/// commands ---------------------------------------------------------------------------------------
	        void  setModelToClip( const Matrix4f& );


/// queries ----------------------------------------------------------------------------------------
	        const Matrix4f& getWorldToClip()                                                  const;

	        //void  transformModelToScreen( const Vector3f&,
	        //                              Vector3f& )                                         const;
	        void  transformModelToScreen( ClipFlags,
	                                      const Vector3f&,
	                                      Vector3f&,
	                                      ClipFlags& )                                        const;
	        void  transformModelToWorld ( const Vector3f&,
	                                      Vector3f& )                                         const;
	        void  transformWorldToClip  ( const Vector3f&,
	                                      Vector4f& )                                         const;


/// fields -----------------------------------------------------------------------------------------
private:
	Matrix4f worldToClip_m;
	Matrix4f modelToClip_m;
	Matrix4f modelToWorld_m;
	Vector2f clipToScreen_m;
};


}}}




#endif//Projector_h
