/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "RasterizationVertex.h"

#include "RasterizationTriangleIm.h"   /// own header is included last


using namespace perceptuum2::transport::projection;




/// standard object services -----------------------------------------------------------------------
RasterizationTriangleIm::RasterizationTriangleIm
(
	const TriangleImplicit& triangle,
	const dword             id
)
 :	TriangleImplicit( triangle ),
	id_m            ( id )
{
}


RasterizationTriangleIm::~RasterizationTriangleIm()
{
}


RasterizationTriangleIm::RasterizationTriangleIm
(
	const RasterizationTriangleIm& other
)
 :	TriangleImplicit( other ),
	id_m            ( other.id_m )
{
}


RasterizationTriangleIm& RasterizationTriangleIm::operator=
(
	const RasterizationTriangleIm& other
)
{
	if( &other != this )
	{
		TriangleImplicit::operator=( other );
		id_m = other.id_m;
	}

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
void RasterizationTriangleIm::set
(
	const TriangleImplicit& triangle,
	const dword             id
)
{
	TriangleImplicit::operator=( triangle );
	id_m = id;
}




/// queries ----------------------------------------------------------------------------------------
bool RasterizationTriangleIm::isFrontFacing
(
	const RasterizationVertex*const pVertexs[3]
) const
{
	const Vector3f*const vertScreenPoss[3] = { &(pVertexs[0]->getScreenPosition()),
	                                           &(pVertexs[1]->getScreenPosition()),
	                                           &(pVertexs[2]->getScreenPosition()) };
	return ( ( (vertScreenPoss[2]->getX() - vertScreenPoss[0]->getX()) * (vertScreenPoss[1]->getY() - vertScreenPoss[0]->getY()) ) <
			 ( (vertScreenPoss[2]->getY() - vertScreenPoss[0]->getY()) * (vertScreenPoss[1]->getX() - vertScreenPoss[0]->getX()) ) );
}
