/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RasterizationVertex_h
#define RasterizationVertex_h


#include "Primitives.h"
#include "Vector3f.h"
#include "ClipFlags.h"

#include "perceptuum2_modelling.h"




#include "perceptuum2_transport_projection.h"
namespace perceptuum2 { namespace transport { namespace projection {
	using namespace hxa7241::primitives;
	using perceptuum2::modelling::Vertex;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::ClipFlags;


class RasterizationVertex
{
/// standard object services -----------------------------------------------------------------------
public:
	         RasterizationVertex();
	         RasterizationVertex( const Vertex&    modelVertex,
	                              const Vector3f&  meshPosition,
	                              const Vector3f&  meshScale,
	                              ClipFlags        objectClipflags,
	                              const Projector& );

	        ~RasterizationVertex();
	         RasterizationVertex( const RasterizationVertex& );
	RasterizationVertex& operator=( const RasterizationVertex& );


/// commands ---------------------------------------------------------------------------------------
	        //void  set( const Vertex&    modelVertex,
	        //           const Vector3f&  meshPosition,
	        //           const Vector3f&  meshScale,
	        //           const Projector& );
	        void  set( const Vertex&    modelVertex,
	                   const Vector3f&  meshPosition,
	                   const Vector3f&  meshScale,
	                   ClipFlags        objectClipflags,
	                   const Projector& );


/// queries ----------------------------------------------------------------------------------------
	        const Vector3f& getWorldPosition()  const;
	        const Vector3f& getScreenPosition() const;
	        ClipFlags       getClipFlags()      const;


/// fields -----------------------------------------------------------------------------------------
private:
	Vector3f  worldPosition_m;
	Vector3f  screenPosition_m;
	//float     screenW_m; ?
	ClipFlags clipFlags_m;
};








/// inlines ///


/// queries ----------------------------------------------------------------------------------------
inline
const Vector3f& RasterizationVertex::getWorldPosition() const
{
	return worldPosition_m;
}


inline
const Vector3f& RasterizationVertex::getScreenPosition() const
{
	return screenPosition_m;
}


inline
ClipFlags RasterizationVertex::getClipFlags() const
{
	return clipFlags_m;
}


}}}




#endif//RasterizationVertex_h
