/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Win32Thread.h"
#include "perceptuum2.rc.h"
#include "stdwin.h"

#include "RenderDesktopWin32.h"


using namespace perceptuum2::platform;




/// standard object services -----------------------------------------------------------------------
RenderDesktopWin32::RenderDesktopWin32
(
	const string& filesPathname,
	HWND          uiHWnd
)
 :	RenderDesktop( hxa7241::win32::Win32ThreadFactory(), filesPathname ),
	uiHWnd_m( uiHWnd )
{
}


RenderDesktopWin32::~RenderDesktopWin32()
{
}




/// overrides --------------------------------------------------------------------------------------
void RenderDesktopWin32::attendToLineDone
(
	const dword lineNumber
)
{
	RenderDesktop::attendToLineDone( lineNumber );

	sendMessageToWindow( MSG_LINE_UPDATE, lineNumber );
}


void RenderDesktopWin32::attendToFrameDone
(
	const bool shouldRepaint
)
{
	RenderDesktop::attendToFrameDone( shouldRepaint );

	sendMessageToWindow( MSG_FRAME_UPDATE, static_cast<dword>(shouldRepaint) );
}


void RenderDesktopWin32::attendToRenderDone
(
	const bool shouldRepaint
)
{
	RenderDesktop::attendToRenderDone( shouldRepaint );

	sendMessageToWindow( MSG_RENDER_UPDATE, static_cast<dword>(shouldRepaint) );
}




/// implementation ---------------------------------------------------------------------------------
void RenderDesktopWin32::sendMessageToWindow
(
	const udword message,
	const dword  param
) const
{
	string statusLine;
	getStatusLine( statusLine );

	::SendMessage( uiHWnd_m,
	               message,
	               reinterpret_cast<WPARAM>(statusLine.c_str()),
	               static_cast<LPARAM>(param) );
	//::PostThreadMessage( uiThreadId_m, MSG_LINE_UPDATE, 0, 0 );
}
