/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "perceptuum2.rc.h"

#include "AboutDialog.h"   /// own header is included last


using namespace perceptuum2::platform_ui;




/// standard object services -----------------------------------------------------------------------
AboutDialog::AboutDialog
(
	HINSTANCE hInstance,
	HWND      hWndParent,
	HDC       dc
)
 :	Win32Dialog( hInstance, hWndParent ),
	hImage_m( 0 ),
	dc_m( 0 ),
	defaultBitmap_m( 0 )
{
	/// load bitmap resource
	hImage_m = ::LoadBitmap( hInstance_m, MAKEINTRESOURCE( IDB_TITLE_IMAGE ) );
	if( hImage_m != 0 )
	{
		/// make DC
		dc_m = ::CreateCompatibleDC( dc );
		if( dc_m != 0 )
		{
			/// put bitmap into DC
			defaultBitmap_m = (HBITMAP)::SelectObject( dc_m, hImage_m );
			if( defaultBitmap_m == 0 )
			{
				::DeleteDC( dc_m );
				::DeleteObject( hImage_m );
				dc_m     = 0;
				hImage_m = 0;
			}
		}
		else
		{
			::DeleteObject( hImage_m );
			hImage_m = 0;
		}
	}
}


AboutDialog::~AboutDialog()
{
	::SelectObject( dc_m, defaultBitmap_m );
	::DeleteObject( hImage_m );
	::DeleteDC( dc_m );
}




/// commands ---------------------------------------------------------------------------------------
int AboutDialog::run()
{
	int dlgReturn;

	/// check if image loaded
	if( hImage_m != NULL )
	{
		/// do dialog
		dlgReturn = Win32Dialog::run();
	}
	else
	{
		/// do message box
		TCHAR buf1[64];
		const int c1 = ::LoadString( hInstance_m, IDS_ABOUT_MESSAGE, (LPSTR)&buf1, sizeof(buf1)/sizeof(TCHAR) );
		TCHAR buf2[16];
		const int c2 = ::LoadString( hInstance_m, IDS_ABOUT_TITLE,   (LPSTR)&buf2, sizeof(buf2)/sizeof(TCHAR) );

		if( (c1 != 0) && (c2 != 0) )
		{
			::MessageBox( hWndParent_m, buf1, buf2, MB_OK | MB_ICONINFORMATION );
		}
		dlgReturn = FALSE;
	}

	return dlgReturn;
}




/// overrides --------------------------------------------------------------------------------------
int AboutDialog::getDialogResourceId() const
{
	return IDD_ABOUTBOX;
}


bool AboutDialog::dialogProc
(
	HWND   hWndDlg,
	UINT   message,
	WPARAM wParam,
	LPARAM //lParam
)
{
	bool didHandleMessage = true;

	switch( message )
	{
	case WM_INITDIALOG :
		{
			/// get image width height
			BITMAP bm;
			bool ok = bool( ::GetObject( hImage_m, sizeof(bm), &bm ) );

			/// get desktop width height
			HWND dw = ::GetDesktopWindow();
			RECT screen;
			ok &= bool( ::GetClientRect( dw, &screen ) );

			/// get window border size
			RECT cr;
			RECT wr;
			ok &= bool( ::GetClientRect( hWndDlg, &cr ) );
			ok &= bool( ::GetWindowRect( hWndDlg, &wr ) );
			const POINT border = { (wr.right - wr.left) - cr.right,  (wr.bottom - wr.top) - cr.bottom };

			/// set width height and position at center screen
			if( ok )
			{
				const POINT pos  = { (screen.right - bm.bmWidth - border.x) / 2,  (screen.bottom - bm.bmHeight - border.y) / 2 };
				const POINT size = { bm.bmWidth + border.x, bm.bmHeight + border.y };
				ok &= bool( ::MoveWindow( hWndDlg, pos.x, pos.y, size.x, size.y, FALSE ) );
			}

			if( !ok )
			{
				::EndDialog( hWndDlg, FALSE );
			}

			break;
		}
	case WM_PAINT :
		{
			PAINTSTRUCT ps;
			HDC targetDc = ::BeginPaint( hWndDlg, &ps );

			::BitBlt( targetDc,
			          ps.rcPaint.left, ps.rcPaint.top,
			          ps.rcPaint.right - ps.rcPaint.left, ps.rcPaint.bottom - ps.rcPaint.top,
					  dc_m,
			          ps.rcPaint.left, ps.rcPaint.top,
					  SRCCOPY );

			::EndPaint( hWndDlg, &ps );

			break;
		}
	case WM_COMMAND :
		{
			if( wParam == IDCANCEL )
			{
				::EndDialog( hWndDlg, TRUE );
				break;
			}
		}
	case WM_LBUTTONDOWN :
		{
			::EndDialog( hWndDlg, TRUE );
			break;
		}
	default:
		didHandleMessage = false;
	}

	return didHandleMessage;
}
