/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ButtonBar_h
#define ButtonBar_h


#include "Primitives.h"
#include "stdwin.h"




#include "perceptuum2_platform_ui.h"
namespace perceptuum2 { namespace platform_ui {
	using namespace hxa7241::primitives;


class ButtonBar
{
/// standard object services -----------------------------------------------------------------------
public:
	         ButtonBar( HWND hWndParent );

	        ~ButtonBar();
private:
	         ButtonBar( const ButtonBar& );
	ButtonBar& operator=( const ButtonBar& );


/// commands ---------------------------------------------------------------------------------------
public:
	        bool enableButton( int id, bool enableOrNot ) const;
	        bool enableButtonsForNoRender()               const;
	        bool enableButtonsForStart()                  const;
	        bool enableButtonsForStop()                   const;

	        void onSize( WPARAM, LPARAM );
	        bool onCommand( WPARAM, Perceptuum2& );
	        bool onCommandKey( WPARAM, Perceptuum2& );


/// queries ----------------------------------------------------------------------------------------
public:
	        bool getWindowRect( RECT& )    const;
	        bool isButtonEnabled( int id ) const;


/// implementation ---------------------------------------------------------------------------------
private:
	static  HINSTANCE getHInstance( HWND );
	static  HWND      createButtonBar( HWND hWndParent, HINSTANCE hInstance );
	        bool      enableButtons( const int ids[], const bool enableOrNots[], int len ) const;

	        void      onStart( Perceptuum2& );
	        void      onStop( Perceptuum2& );
	        void      onSave( Perceptuum2& );
	        void      onGlare( Perceptuum2& );
	        void      onTone( Perceptuum2& );
	        void      onInfo( Perceptuum2& );
	        void      onAbout();


/// fields -----------------------------------------------------------------------------------------
private:
	HINSTANCE hInstance_m;
	HWND      hWnd_m;

	static const char CLASS_NAME[];
	static const char KEYS[];
	enum { START_KEY, STOP_KEY, SAVE_KEY, GLARE_KEY, TONEMAP_KEY, INFO_KEY, ABOUT_KEY };
};


}}




#endif//ButtonBar_h
