/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ArrayFixed_h
#define ArrayFixed_h


#include "Primitives.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * a simple wrapper for a stack array.<br/><br/>
 *
 * each instance of different length is a different type.<br/><br/>
 *
 * operator[] wraps around out of bound indexs.
 */
template<class TYPE, int LENGTH>
class ArrayFixed
{
/// standard object services -----------------------------------------------------------------------
public:
	               ArrayFixed();
	explicit       ArrayFixed( bool isZeroed );

	              ~ArrayFixed();
	               ArrayFixed( const ArrayFixed& );
	        ArrayFixed& operator=( const ArrayFixed& );


/// commands ---------------------------------------------------------------------------------------
	        TYPE*  getMemory();
	        TYPE&  operator[]( int index );               /// wraps out of bounds

	        void   zero();


/// queries ----------------------------------------------------------------------------------------
	        dword  getLength()                   const;

	        const TYPE*  getMemory()             const;
	        const TYPE&  operator[]( int index ) const;   /// wraps out of bounds


/// fields -----------------------------------------------------------------------------------------
private:
	TYPE pMemory_m[LENGTH];
};








/// standard object services -----------------------------------------------------------------------
template<class TYPE, int LENGTH>
inline
ArrayFixed<TYPE, LENGTH>::ArrayFixed()
{
}


template<class TYPE, int LENGTH>
inline
ArrayFixed<TYPE, LENGTH>::ArrayFixed
(
	const bool isZeroed
)
{
	if( isZeroed )
	{
		zero();
	}
}


template<class TYPE, int LENGTH>
inline
ArrayFixed<TYPE, LENGTH>::~ArrayFixed()
{
}


template<class TYPE, int LENGTH>
inline
ArrayFixed<TYPE, LENGTH>::ArrayFixed
(
	const ArrayFixed<TYPE, LENGTH>& other
)
{
	ArrayFixed::operator=( other );
}


template<class TYPE, int LENGTH>
ArrayFixed<TYPE, LENGTH>& ArrayFixed<TYPE, LENGTH>::operator=
(
	const ArrayFixed<TYPE, LENGTH>& other
)
{
	for( int i = LENGTH;  i-- > 0; )
	{
		pMemory_m[i] = other.pMemory_m[i];
	}

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
template<class TYPE, int LENGTH>
inline
TYPE* ArrayFixed<TYPE, LENGTH>::getMemory()
{
	return pMemory_m;
}


template<class TYPE, int LENGTH>
TYPE& ArrayFixed<TYPE, LENGTH>::operator[]
(
	const int index
)
{
	return pMemory_m[ (index >= 0 ? index : -index) % LENGTH ];
}


template<class TYPE, int LENGTH>
void ArrayFixed<TYPE, LENGTH>::zero()
{
	for( dword i = LENGTH;  i-- > 0; )
	{
		pMemory_m[i] = TYPE();
	}
}




/// queries ----------------------------------------------------------------------------------------
template<class TYPE, int LENGTH>
inline
dword ArrayFixed<TYPE, LENGTH>::getLength() const
{
	return LENGTH;
}


template<class TYPE, int LENGTH>
inline
const TYPE* ArrayFixed<TYPE, LENGTH>::getMemory() const
{
	return pMemory_m;
}


template<class TYPE, int LENGTH>
const TYPE& ArrayFixed<TYPE, LENGTH>::operator[]
(
	const int index
) const
{
	return pMemory_m[ (index >= 0 ? index : -index) % LENGTH ];
}


}}




#endif//ArrayFixed_h
