/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Logger_h
#define Logger_h


#include "Primitives.h"
#include "LogThreadSafe.h"
#include <string>
#include <sstream>
//#include <cstring>




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::string;
	using std::ostringstream;
	typedef const char* constcharptr;


/**
 * a buffer to the log.<br/><br/>
 *
 * writes out to the log on destruction, and appends a '\n'.<br/>
 * usage example:<br/>
 * <code>hxa7241::utility::Logger() << "\tindex: " << i << "  value: " << items[i];</code>
 *
 * @see LogThreadSafe
 */
class Logger
{
/// standard object services -----------------------------------------------------------------------
public:
	         Logger();
	         Logger( dword fpPrecision,
	                 bool  isFpScientific );
	         Logger( bool  isFpScientific,
	                 dword fpPrecision );
	explicit Logger( dword fpPrecision );
	explicit Logger( bool  isFpScientific );

	        ~Logger();
private:
	         Logger( const Logger& );
	Logger&  operator=( const Logger& );


/// commands ---------------------------------------------------------------------------------------
public:
	template<class TYPE>
	        Logger& operator<<( const TYPE& );


/// queries ----------------------------------------------------------------------------------------
	/**
	 * attention mid level char sequence.
	 */
	static const char EXC1[];
	/**
	 * attention high level char sequence.
	 */
	static const char EXC2[];
	/**
	 * blank line char sequence.
	 */
	static const char BRK[];
	/**
	 * dividing rule char sequence.
	 */
	static const char DIV[];
//	static const char* EXC1();
//	static const char* EXC2();
//	static const char* BRK();
//	static const char* DIV();


/// implementation ---------------------------------------------------------------------------------
private:
	        void init( dword fpPrecision,
	                   bool  isFpScientific );
	        void flush();


/// fields -----------------------------------------------------------------------------------------
private:
	ostringstream buffer_m;

};








/// TEMPLATES ///


/// commands ---------------------------------------------------------------------------------------
template<class TYPE>
Logger& Logger::operator<<( const TYPE& a )
{
	if( LogThreadSafe::isEnabled() )
	{
		buffer_m << a;
	}
	return *this;
}


/// specialisations
template<>
Logger& Logger::operator<< <constcharptr>( const constcharptr& a )
{
	if( LogThreadSafe::isEnabled() )
	{
		if( a != 0 )
		{
			buffer_m << a;

//			/// flush if theres a newline char in the string
//			if( std::strrchr( a, '\n' ) != 0 )
//			{
//				flush();
//			}
		}
		else
		{
			buffer_m << " nullcp ";
		}
	}
	return *this;
}


template<>
Logger& Logger::operator<< <string>( const string& a )
{
	if( LogThreadSafe::isEnabled() )
	{
		operator<<( a.c_str() );
	}
	return *this;
}




}}




#endif//Logger_h
