/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RandomFast_h
#define RandomFast_h


#include "Primitives.h"
#include "Random.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * simplest fastest random number generator.<br/><br/>
 *
 * gives a signed dword in the range [MIN_VALUE, MAX_VALUE], and a float in the
 * range 0 to 1 (i would think [0, 1) but not sure...).<br/><br/>
 *
 * not very random, but it is fast.<br/><br/>
 *
 * adapted from: <cite>'Numerical Recipes In C', ch7 p284-285, www.nr.com.</cite>
 */
class RandomFast
	: public Random
{
public:
/// standard object services -----------------------------------------------------------------------
	explicit    RandomFast( const dword seed =0 )
	 :	Random(),
		random_m( seed )
	{
	}

	virtual    ~RandomFast()
	{
	}

	            RandomFast( const RandomFast& other )
	 :	Random( other ),
		random_m( other.random_m )
	{
	}

	RandomFast& operator=( const RandomFast& other )
	{
		Random::operator=( other );
		random_m = other.random_m;

		return *this;
	}


/// commands ---------------------------------------------------------------------------------------
	virtual void  setSeed( const dword seed )
	{
		random_m = seed;
	}

	virtual void  next()
	{
		random_m = dword(1664525) * random_m + dword(1013904223);
	}


/// queries ----------------------------------------------------------------------------------------
	virtual dword getDword() const
	{
		return random_m;
	}

	virtual float getFloat() const
	{
		dword itemp = dword(0x3F800000) | (dword(0x007FFFFF) & random_m);
		return *(reinterpret_cast<float*>(&itemp)) - 1.0f;
	}


/// fields -----------------------------------------------------------------------------------------
private:
	/// current value of sequence and seed of the following part of the sequence
	dword random_m;
};


}}




/*
test sequence of 32-bit values (hex):
00000000, 3C6EF35F, 47502932, D1CCF6E9,
AAF95334, 6252E503, 9F2EC686, 57FE6C2D,
A3D95FA8, 81FDBEE7, 94F0AF1A, CBF633B1
*/




#endif//RandomFast_h
