/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "TextSerializerOut.h"   /// own header is included last


using namespace hxa7241::utility;




/// statics
const char TextSerializerOut::CLASS_NAME[] = "hxa7241::utility::TextSerializerOut";




/// standard object services -----------------------------------------------------------------------
TextSerializerOut::TextSerializerOut
(
	ostream& out
)
 :	XmlSerializerOut( out )
{
}


TextSerializerOut::~TextSerializerOut()
{
}




/// implementation ---------------------------------------------------------------------------------
void TextSerializerOut::writeHeader
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	out_m << NEWLINE;
}


void TextSerializerOut::writeFooter
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	out_m << NEWLINE;
}


void TextSerializerOut::writePreObject
(
	const char* name,
	const char* ,//typeName,
	const char* //comment
)
{
	out_m << NEWLINE;
	writeIndent( indent_m );

	out_m << name;
	out_m << ":  ";

	++indent_m;
}


void TextSerializerOut::writePostObject
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	/// decrease indent
	--indent_m;

	/// write formatting (if last content wasnt primitive type)
	if( !wasPrimitive_m )
	{
		out_m << NEWLINE;
		writeIndent( indent_m );
	}
	wasPrimitive_m = false;
}


void TextSerializerOut::writeString
(
	const char* pString
) const
{
	if( pString != 0 )
	{
		out_m << pString;
	}
}
