/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef UriFile_h
#define UriFile_h


#include "Primitives.h"
#include <string>




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::string;


/**
 * a uri for a file, ie with 'file://' scheme.<br/><br/>
 *
 * analyses a plain string, so name, path and pathname are accessible separately.
 * <br/><br/>
 *
 * supports unix or win path separators.<br/><br/>
 *
 * eg (unix mode):<br/>
 * <pre>"file://c/perceptuum2/models/table.scene.p2.xml"
 * 	  scheme:   file
 * 	  path:     c/perceptuum2/models/
 * 	  filename: table.scene.p2.xml</pre>
 */
class UriFile
{
/// standard object services -----------------------------------------------------------------------
public:
	         UriFile();
	explicit UriFile( const string& );
	explicit UriFile( const char[] );

	        ~UriFile();
	         UriFile( const UriFile& );
	UriFile& operator=( const UriFile& );


/// commands ---------------------------------------------------------------------------------------
public:
	enum EMode { UNIX, WIN };
	        void  setMode( EMode );


/// queries ----------------------------------------------------------------------------------------
public:
	static  const char* getScheme();
	static  const char* getSchemeWithSeparator();
	static  const char* getSchemeSeparator();

	        bool  isValid()              const;
	        void  getString( string& )   const;

	        void  getPath( string& )     const;
	        void  getName( string& )     const;
	        void  getPathname( string& ) const;

	        EMode getMode()              const;


/// implementation ---------------------------------------------------------------------------------
private:
	        void  validate();
	        void  modulate( EMode, string& ) const;


/// fields -----------------------------------------------------------------------------------------
private:
	/// "file://c/perceptuum2/models/table.scene.p2.xml"
	///		scheme:   file
	///		path:     c/perceptuum2/models/
	///		filename: table.scene.p2.xml

	string string_m;
	EMode  mode_m;
	dword  pathPos_m;
	dword  namePos_m;

	static const EMode DEFAULT_MODE = WIN;

	static const char  SCHEME[];
	static const char  SCHEME_SEP[];
	static const char  SCHEME_AND_SEP[];
	static const char  SEPARATORS[];
};


}}




#endif//UriFile_h
