/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Win32Thread_h
#define Win32Thread_h


#include "Primitives.h"
#include "Thread.h"
#include "stdwin.h"




#include "hxa7241_win32.h"
namespace hxa7241 { namespace win32 {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Thread;
	using hxa7241::utility::ThreadFactory;
	using hxa7241::utility::Runnable;
	using hxa7241::utility::ThreadLock;


class Win32Thread
 : public Thread
{
/// standard object services -----------------------------------------------------------------------
public:
	             Win32Thread( Runnable* );

	virtual     ~Win32Thread();
private:
	             Win32Thread( const Win32Thread& );
	Win32Thread& operator=( const Win32Thread& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  start();
	virtual void  stopForcibly();
	virtual void  join();
	virtual void  setPriority( dword minus3toPlus3 );
	virtual void  suspend();
	virtual void  resume();


/// queries ----------------------------------------------------------------------------------------
public:
	virtual bool  isAlive()             const;
	virtual dword getPriorityRange()    const;

	virtual ThreadLock* createLock()    const;


/// fields -----------------------------------------------------------------------------------------
private:
	HANDLE threadHandle_m;
	DWORD  threadId_m;

	static const int  PRIORITIES_VALUES[];
	static const int  PRIORITIES_RANGE;
	static const int* PRIORITIES;

	static const char CLASS_NAME[];
	static const char EXCEPTION_MESSAGE1[];
	static const char EXCEPTION_MESSAGE2[];
};




class Win32ThreadFactory
 : public ThreadFactory
{
/// standard object services -----------------------------------------------------------------------
public:
	         Win32ThreadFactory() {}

	virtual ~Win32ThreadFactory() {}
private:
	         Win32ThreadFactory( const Win32ThreadFactory& );
	Win32ThreadFactory& operator=( const Win32ThreadFactory& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual Thread* createThread( Runnable* r ) const
	{
		return new Win32Thread( r );
	}
};


}}




#endif//Win32Thread_h
